@@ -2,13 +2,13 @@
 import logging
 from typing import Optional
 
-from pytype import overlay
 from pytype import special_builtins
 from pytype import utils
 from pytype.abstract import abstract
 from pytype.abstract import abstract_utils
 from pytype.abstract import function
 from pytype.abstract import mixin
+from pytype.overlays import overlay
 from pytype.typegraph import cfg
 
 log = logging.getLogger(__name__)
@@ -286,28 +286,6 @@ class AbstractAttributeHandler(utils.ContextWeakrefMixin):
         # Fall back to __getattr__ if the attribute doesn't otherwise exist.
         node, attr = self._get_attribute_computed(
             node, cls, name, valself, compute_function="__getattr__")
-    for base in obj.mro:
-      if not isinstance(base, abstract.InterpreterClass):
-        break
-      annots = abstract_utils.get_annotations_dict(base.members)
-      if annots:
-        typ = annots.get_type(node, name)
-        if not typ:
-          continue
-        if typ.formal and valself:
-          # The attribute contains a class-scoped type parameter, so we need to
-          # reinitialize it with the current instance's parameter values.
-          subst = abstract_utils.get_type_parameter_substitutions(
-              valself.data, self.ctx.annotation_utils.get_type_parameters(typ))
-          typ = self.ctx.annotation_utils.sub_one_annotation(
-              node, typ, [subst], instantiate_unbound=False)
-          _, attr = self.ctx.annotation_utils.init_annotation(node, name, typ)
-        elif attr is None:
-          # An attribute has been declared but not defined, e.g.,
-          #   class Foo:
-          #     bar: int
-          _, attr = self.ctx.annotation_utils.init_annotation(node, name, typ)
-        break
     if attr is not None:
       attr = self._filter_var(node, attr)
     return node, attr
@@ -414,6 +392,42 @@ class AbstractAttributeHandler(utils.ContextWeakrefMixin):
                                       function.Args((name_var,)))
     return node, None
 
+  def _lookup_from_mro_flat(self, node, base, name, valself, skip):
+    """Look for an identifier in a single base class from an MRO."""
+    # Potentially skip part of MRO, for super()
+    if base in skip:
+      return None
+    # Check if the attribute is declared via a variable annotation.
+    typ = None
+    if isinstance(base, abstract.InterpreterClass):
+      annots = abstract_utils.get_annotations_dict(base.members)
+      if annots:
+        typ = annots.get_type(node, name)
+        if typ and typ.formal and valself:
+          # The attribute contains a class-scoped type parameter, so we need to
+          # reinitialize it with the current instance's parameter values. In
+          # this case, we use the type from the annotation regardless of whether
+          # the attribute is otherwise defined.
+          subst = abstract_utils.get_type_parameter_substitutions(
+              valself.data, self.ctx.annotation_utils.get_type_parameters(typ))
+          typ = self.ctx.annotation_utils.sub_one_annotation(
+              node, typ, [subst], instantiate_unbound=False)
+          _, attr = self.ctx.annotation_utils.init_annotation(node, name, typ)
+          return attr
+    # When a special attribute is defined on a class buried in the MRO,
+    # get_attribute (which calls get_special_attribute) is never called on
+    # that class, so we have to call get_special_attribute here as well.
+    var = base.get_special_attribute(node, name, valself)
+    if var is None:
+      node, var = self._get_attribute_flat(node, base, name, valself)
+    if var is None or not var.bindings:
+      # If the attribute is undefined, we use the annotated type, if any.
+      if typ:
+        _, attr = self.ctx.annotation_utils.init_annotation(node, name, typ)
+        return attr
+      return None
+    return var
+
   def _lookup_from_mro(self, node, cls, name, valself, skip):
     """Find an identifier in the MRO of the class."""
     if isinstance(cls, (abstract.Unknown, abstract.Unsolvable)):
@@ -423,16 +437,8 @@ class AbstractAttributeHandler(utils.ContextWeakrefMixin):
     ret = self.ctx.program.NewVariable()
     add_origins = [valself] if valself else []
     for base in cls.mro:
-      # Potentially skip part of MRO, for super()
-      if base in skip:
-        continue
-      # When a special attribute is defined on a class buried in the MRO,
-      # get_attribute (which calls get_special_attribute) is never called on
-      # that class, so we have to call get_special_attribute here as well.
-      var = base.get_special_attribute(node, name, valself)
+      var = self._lookup_from_mro_flat(node, base, name, valself, skip)
       if var is None:
-        node, var = self._get_attribute_flat(node, base, name, valself)
-      if var is None or not var.bindings:
         continue
       for varval in var.bindings:
         value = varval.data
@@ -444,9 +450,8 @@ class AbstractAttributeHandler(utils.ContextWeakrefMixin):
               not isinstance(base, abstract.InterpreterClass)):
             # See BaseValue.property_get for an explanation of the
             # parameters we're passing here.
-            value = value.property_get(
-                valself.AssignToNewVariable(node),
-                abstract_utils.is_subclass(valself, cls))
+            value = value.property_get(valself.AssignToNewVariable(node),
+                                       abstract_utils.is_subclass(valself, cls))
           if isinstance(value, abstract.Property):
             node, value = value.call(node, None, None)
             final_values = value.data
@@ -477,9 +482,16 @@ class AbstractAttributeHandler(utils.ContextWeakrefMixin):
   def _get_member(self, node, obj, name, valself):
     """Get a member of an object."""
     if isinstance(obj, mixin.LazyMembers):
-      if valself and isinstance(valself.data, abstract.ParameterizedClass):
+      if not valself:
+        subst = None
+      elif isinstance(valself.data, abstract.ParameterizedClass):
         subst = {f"{valself.data.full_name}.{k}": v.instantiate(node)
                  for k, v in valself.data.formal_type_parameters.items()}
+      elif isinstance(valself.data, abstract.Instance):
+        # We need to rebind the parameter values at the root because that's the
+        # node at which load_lazy_attribute() converts pyvals.
+        subst = {k: self.ctx.program.NewVariable(v.data, [], self.ctx.root_node)
+                 for k, v in valself.data.instance_type_parameters.items()}
       else:
         subst = None
       obj.load_lazy_attribute(name, subst)
@@ -606,7 +618,7 @@ class AbstractAttributeHandler(utils.ContextWeakrefMixin):
         sig.set_annotation("self", sig.annotations[self_name])
         sig.del_annotation(self_name)
         sig.param_names = ("self",) + sig.param_names[1:]
-        new_val = abstract.SimpleFunction.from_signature(sig, self.ctx)
+        new_val = abstract.SimpleFunction(sig, self.ctx)
         var2.AddBinding(new_val, {b}, node)
       var = var2
 
