@@ -5,7 +5,6 @@ import types
 
 from pytype import blocks
 from pytype import datatypes
-from pytype import overlay_dict
 from pytype import module_utils
 from pytype import special_builtins
 from pytype import utils
@@ -13,6 +12,8 @@ from pytype.abstract import abstract
 from pytype.abstract import abstract_utils
 from pytype.abstract import mixin
 from pytype.overlays import attr_overlay
+from pytype.overlays import overlay_dict
+from pytype.overlays import named_tuple
 from pytype.overlays import typed_dict
 from pytype.overlays import typing_overlay
 from pytype.pyc import loadmarshal
@@ -273,6 +274,15 @@ class Converter(utils.ContextWeakrefMixin):
     builder = typed_dict.TypedDictBuilder(ctx)
     return builder.make_class_from_pyi(name, pytd_cls)
 
+  def make_namedtuple_builder(self, ctx):
+    """Make a namedtuple builder."""
+    return named_tuple.NamedTupleClassBuilder(ctx)
+
+  def make_namedtuple(self, name, pytd_cls, ctx):
+    """Make a namedtuple class from a pytd class."""
+    builder = named_tuple.NamedTupleClassBuilder(ctx)
+    return builder.make_class_from_pyi(name, pytd_cls)
+
   def get_maybe_abstract_instance(self, data):
     """Get an instance of the same type as the given data, abstract if possible.
 
