@@ -14,6 +14,7 @@ from pytype.abstract import class_mixin
 from pytype.abstract import function
 from pytype.overlays import attr_overlay
 from pytype.overlays import dataclass_overlay
+from pytype.overlays import named_tuple
 from pytype.overlays import typed_dict
 from pytype.overlays import typing_overlay
 from pytype.pyi import metadata
@@ -413,7 +414,11 @@ class Converter(utils.ContextWeakrefMixin):
       assert name != v.name
       return pytd.Alias(name, pytd.NamedType(v.name))
     elif isinstance(v, abstract.InterpreterClass):
-      if v.official_name is None or name == v.official_name:
+      if v.module:  # alias to an imported type
+        return pytd.Constant(
+            name, pytd.GenericType(pytd.NamedType("builtins.type"),
+                                   (pytd.NamedType(v.full_name),)))
+      elif v.official_name is None or name == v.official_name:
         return self._class_to_def(node, v, name)
       else:
         return pytd.Alias(name, pytd.NamedType(v.official_name))
@@ -689,9 +694,12 @@ class Converter(utils.ContextWeakrefMixin):
           # input type, which pytype struggles to reason about.
           method = cast(pytd.Function,
                         self.value_to_pytd_def(node, value, name))
-          keep = lambda name: not name or name.startswith(v.name)
+          def keep(self_type):
+            maybe_params = pytd_utils.UnpackGeneric(self_type, "builtins.type")
+            name = maybe_params[0].name if maybe_params else self_type.name
+            return not name or name.startswith(v.name)
           signatures = tuple(s for s in method.signatures
-                             if not s.params or keep(s.params[0].type.name))
+                             if not s.params or keep(s.params[0].type))
           if signatures and signatures != method.signatures:
             # Filter out calls made from subclasses unless they are the only
             # ones recorded; when inferring types for ParentClass.__init__, we
@@ -783,8 +791,23 @@ class Converter(utils.ContextWeakrefMixin):
         del methods[name]
         constants[name].add_type(pytd.AnythingType())
 
-    constants = [pytd.Constant(name, builder.build())
-                 for name, builder in constants.items() if builder]
+    if isinstance(v, named_tuple.NamedTupleClass):
+      # Filter out generated members from namedtuples
+      cls_bases = v.props.bases
+      fieldnames = [x.name for x in v.props.fields]
+      methods = {k: m for k, m in methods.items()
+                 if k not in v.generated_members}
+      constants = {k: c for k, c in constants.items()
+                   if k not in v.generated_members}
+      for k, c in constants.items():
+        # If we have added class members to a namedtuple, do not emit them as
+        # regular fields.
+        if k not in fieldnames:
+          c.wrap("typing.ClassVar")
+      slots = None
+    else:
+      cls_bases = v.bases()
+      slots = v.slots
 
     metaclass = v.metaclass(node)
     if metaclass is not None:
@@ -795,7 +818,8 @@ class Converter(utils.ContextWeakrefMixin):
     # into this class's pytd.
     bases = []
     missing_bases = []
-    for basevar in v.bases():
+
+    for basevar in cls_bases:
       if len(basevar.bindings) == 1:
         b, = basevar.data
         if b.official_name is None and isinstance(b, abstract.InterpreterClass):
@@ -806,21 +830,56 @@ class Converter(utils.ContextWeakrefMixin):
         bases.append(pytd_utils.JoinTypes(b.get_instance_type(node)
                                           for b in basevar.data))
 
+    # If a namedtuple was constructed via one of the functional forms, it will
+    # not have a base class. Since we uniformly output all namedtuple classes as
+    # subclasses of typing.NamedTuple we need to add it in here.
+    if isinstance(v, named_tuple.NamedTupleClass):
+      if not any(x.name == "typing.NamedTuple" for x in bases):
+        bases.append(pytd.NamedType("typing.NamedTuple"))
+
+    if any(isinstance(x, named_tuple.NamedTupleClass)
+           for x in missing_bases):
+      # If inheriting from an anonymous namedtuple, mark all derived class
+      # constants as ClassVars, otherwise MergeBaseClasses will convert them
+      # into namedtuple fields.
+      for c in constants.values():
+        c.wrap("typing.ClassVar")
+
+    final_constants = []
+    if isinstance(v, named_tuple.NamedTupleClass):
+      # The most precise way to get defaults is to check v.__new__.__defaults__,
+      # since it's possible for the user to manually set __defaults__. If
+      # retrieving this attribute fails, we fall back to the defaults set when
+      # the class was built.
+      try:
+        new = abstract_utils.get_atomic_value(
+            v.members["__new__"], abstract.SignedFunction)
+      except abstract_utils.ConversionError:
+        fields_with_defaults = {f.name for f in v.props.fields if f.default}
+      else:
+        fields_with_defaults = set(new.signature.defaults)
+    else:
+      fields_with_defaults = set()
+    for name, builder in constants.items():
+      if not builder:
+        continue
+      value = pytd.AnythingType() if name in fields_with_defaults else None
+      final_constants.append(pytd.Constant(name, builder.build(), value))
+
     # Collect nested classes
     # TODO(mdemello): We cannot put these in the output yet; they fail in
     # load_dependencies because of the dotted class name (google/pytype#150)
     classes = [self._class_to_def(node, x, x.name)
                for x in v.get_inner_classes()]
     classes = [x.Replace(name=class_name + "." + x.name) for x in classes]
-
     cls = pytd.Class(name=class_name,
                      metaclass=metaclass,
                      bases=tuple(bases),
                      methods=tuple(methods.values()),
-                     constants=tuple(constants),
+                     constants=tuple(final_constants),
                      classes=(),
                      decorators=tuple(decorators),
-                     slots=v.slots,
+                     slots=slots,
                      template=())
     for base in missing_bases:
       base_cls = self.value_to_pytd_def(node, base, base.name)
