@@ -11,12 +11,12 @@
 # frozen anyway we needn't bother about that for now.
 
 
-from pytype import overlay
 from pytype.abstract import abstract
 from pytype.abstract import abstract_utils
 from pytype.abstract import function
 from pytype.overlays import classgen
 from pytype.overlays import dataclass_overlay
+from pytype.overlays import overlay
 from pytype.pytd import pytd
 
 
