@@ -5,15 +5,17 @@ import keyword
 
 from typing import Any, List, Optional
 
-from pytype import overlay_utils
+from pytype import special_builtins
 from pytype import utils
 from pytype.abstract import abstract
 from pytype.abstract import abstract_utils
 from pytype.abstract import class_mixin
 from pytype.abstract import function
 from pytype.overlays import classgen
+from pytype.overlays import overlay_utils
 from pytype.pytd import escape
 from pytype.pytd import pytd
+from pytype.pytd import pytd_utils
 from pytype.pytd import visitors
 
 
@@ -387,7 +389,6 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
 
     f_locals = abstract_utils.get_atomic_python_constant(f_locals)
 
-    # retrieve __qualname__ to get the name of class
     name = f_locals["__qualname__"]
     name = abstract_utils.get_atomic_python_constant(name)
     if "." in name:
@@ -435,6 +436,101 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
 
     return node, cls_var
 
+  def make_class_from_pyi(self, cls_name, pytd_cls):
+    """Make a NamedTupleClass from a pyi class."""
+    # NOTE: Returns the abstract class, not a variable.
+    ctx = self.ctx
+    fields = []
+    classvars = []
+    for c in pytd_cls.constants:
+      ct = c.type.base_type if isinstance(c.type, pytd.Annotated) else c.type
+      if isinstance(ct, pytd.GenericType):
+        cv = pytd_utils.UnpackGeneric(ct, "typing.ClassVar")
+      elif ct.name == "typing.ClassVar":
+        cv = (pytd.AnythingType(),)
+      else:
+        cv = None
+      if cv is not None:
+        typ, = cv
+        classvars.append((c.name, typ))
+      else:
+        # The field types may refer back to the class being built.
+        with ctx.allow_recursive_convert():
+          if c.value:
+            default = ctx.new_unsolvable(ctx.root_node)
+          else:
+            default = None
+          fields.append(
+              Field(c.name, ctx.convert.constant_to_value(ct), default))
+
+    bases = []
+    for x in pytd_cls.bases:
+      b = ctx.convert.constant_to_var(x)
+      if pytd_cls.template:
+        # We need to set the generated class's _template directly here, since we
+        # bypass the normal mechanism to copy a template from a pytd class to a
+        # PyTDClass (see abstract._base._compute_template())
+        tmpl = [ctx.convert.constant_to_value(t.type_param)
+                for t in pytd_cls.template]
+        b.data[0]._template = tmpl  # pylint: disable=protected-access
+      bases.append(b)
+    props = NamedTupleProperties(cls_name, fields, bases)
+    node = ctx.root_node
+    node, cls_var = _build_namedtuple(props, node, ctx)
+    cls = abstract_utils.get_atomic_value(cls_var)
+
+    # Adjust the class after construction
+
+    # Since we might have classvars or methods referring to the class itself,
+    # we need to insert the just-constructed class into the convert cache.
+    ctx.convert._convert_cache[("constant", pytd_cls, type(pytd_cls))] = cls  # pylint: disable=protected-access
+
+    # Set the official class name.
+    cls.update_official_name(cls.name)
+
+    # Convert classvars, which might be recursively typed
+    with ctx.allow_recursive_convert():
+      classvars = [(name, ctx.convert.constant_to_value(typ))
+                   for name, typ in classvars]
+
+    # Add fields and classvars to the annotation dictionary
+    locals_ = {f.name: abstract_utils.Local(node, None, f.typ, None, ctx)
+               for f in fields}
+    locals_.update({name: abstract_utils.Local(node, None, typ, None, ctx)
+                    for name, typ in classvars})
+    annots = abstract.AnnotationsDict(locals_, ctx).to_variable(node)
+    cls.members["__annotations__"] = annots
+
+    # Add classvars to members
+    for name, typ in classvars:
+      _, var = ctx.vm.init_class(ctx.root_node, typ)
+      cls.members[name] = var
+
+    # Add any methods defined in the class, potentially overwriting the
+    # generated ones
+    for m in pytd_cls.methods:
+      # We need to allow recursive conversion because overriding __new__ adds a
+      # typevar that refers back to the class, and reloading the pyi file
+      # triggers a recursion error in convert.
+      with ctx.allow_recursive_convert():
+        # Create a SimpleFunction from the pytd method; storing a PyTDFunction
+        # as an InterpreterClass method does not behave correctly (see
+        # AttributeHandler._lookup_from_mro)
+        sig = function.Signature.from_pytd(ctx, m.name, m.signatures[0])
+        meth = abstract.SimpleFunction(sig, ctx)
+        m_var = meth.to_variable(ctx.root_node)
+        # Handle classmethods and staticmethods
+        # TODO(mdemello): We really need a better way to do this.
+        # Move to SimpleFunction.from_pyi if we add it to one more overlay.
+        args = function.Args(posargs=(m_var,))
+        if m.kind == pytd.MethodKind.CLASSMETHOD:
+          _, m_var = special_builtins.ClassMethod(ctx).call(node, meth, args)
+        elif m.kind == pytd.MethodKind.STATICMETHOD:
+          _, m_var = special_builtins.StaticMethod(ctx).call(node, meth, args)
+        cls.members[m.name] = m_var
+
+    return cls
+
 
 class _DictBuilder:
   """Construct OrderedDict abstract classes for namedtuple members."""
@@ -454,6 +550,30 @@ class _DictBuilder:
     }, self.ctx)
 
 
+class NamedTupleClass(abstract.InterpreterClass):
+  """Named tuple classes."""
+
+  def __init__(self, *args):
+    super().__init__(*args)
+    # Store the original properties, to output to pyi files.
+    self.props = None
+    self.generated_members = None
+
+  def instantiate(self, node, container=None):
+    # For all generic typevars T, add a type parameter alias between cls.T and
+    # path.to.module.cls.T
+    inst = super().instantiate(node, container)
+    for ival in inst.data:
+      cls = ival.cls
+      long = cls.full_name
+      for t in cls.template:
+        short = t.module
+        param = t.name
+        ival.instance_type_parameters.add_alias(
+            f"{short}.{param}", f"{long}.{param}", lambda x, y: x or y)
+    return inst
+
+
 def _build_namedtuple(props, node, ctx):
   """Build an InterpreterClass representing the namedtuple."""
 
@@ -477,7 +597,9 @@ def _build_namedtuple(props, node, ctx):
   odict = _DictBuilder(ctx)
   # __dict__ and _field_defaults are both OrderedDicts of
   # { field_name: field_type_instance }
-  field_dict_cls = odict.make(field_types_union)
+  # The field types may refer back to the class being built.
+  with ctx.allow_recursive_convert():
+    field_dict_cls = odict.make(field_types_union)
   members["__dict__"] = field_dict_cls.instantiate(node)
   members["_field_defaults"] = field_dict_cls.instantiate(node)
 
@@ -505,14 +627,16 @@ def _build_namedtuple(props, node, ctx):
                                          ctx)
 
   params = [Param(f.name, f.typ) for f in props.fields]
-  members["__new__"] = overlay_utils.make_method(
-      ctx,
-      node,
-      name="__new__",
-      self_param=Param("cls", cls_type),
-      params=params,
-      return_type=cls_type_param,
-  )
+  # The parameter types may refer back to the class being built.
+  with ctx.allow_recursive_convert():
+    members["__new__"] = overlay_utils.make_method(
+        ctx,
+        node,
+        name="__new__",
+        self_param=Param("cls", cls_type),
+        params=params,
+        return_type=cls_type_param,
+    )
 
   # __init__
   members["__init__"] = overlay_utils.make_method(
@@ -616,10 +740,10 @@ def _build_namedtuple(props, node, ctx):
   cls_props = class_mixin.ClassBuilderProperties(
       name_var=ctx.convert.build_string(node, props.name),
       bases=final_bases,
-      class_dict_var=cls_dict.to_variable(node))
+      class_dict_var=cls_dict.to_variable(node),
+      class_type=NamedTupleClass,)
   node, cls_var = ctx.make_class(node, cls_props)
   cls = cls_var.data[0]
-
   # Now that the class has been made, we can complete the TypeParameter used
   # by __new__, _make and _replace.
   cls_type_param.bound = cls
@@ -633,5 +757,10 @@ def _build_namedtuple(props, node, ctx):
   node = ctx.attribute_handler.set_attribute(
       node, new_attr, "__defaults__", defaults)
 
+  # Store the original properties
+  cls.props = props
+  cls.generated_members = (
+      set(members.keys()) - set((x.name for x in props.fields)))
+
   ctx.vm.trace_classdef(cls_var)
   return node, cls_var
