@@ -120,7 +120,7 @@ class ParserTest(parser_test_base.ParserTestBase):
 
   def test_constant_pep526(self):
     self.check("x : str", "x: str")
-    self.check("x : str = ...", "x: str")
+    self.check("x : str = ...", "x: str = ...")
 
   def test_alias_or_constant(self):
     self.check("x = True", "x: bool")
@@ -543,6 +543,23 @@ class ParserTest(parser_test_base.ParserTestBase):
       x: typing.NamedTuple("A", []) = ...
     """, 1, e)
 
+  def test_match_args(self):
+    self.check("""
+      from typing import Final
+
+      class A:
+          __match_args__ = ("a", "b")
+
+      class B:
+          __match_args__: Final = ("a", "b")
+    """, """
+      class A:
+          __match_args__: tuple
+
+      class B:
+          __match_args__: tuple
+    """)
+
 
 class QuotedTypeTest(parser_test_base.ParserTestBase):
 
@@ -553,7 +570,7 @@ class QuotedTypeTest(parser_test_base.ParserTestBase):
       y: "List[A]" = ...
     """, """
       x: A
-      y: List[A]
+      y: List[A] = ...
 
       class A: ...
     """)
@@ -676,42 +693,21 @@ class NamedTupleTest(parser_test_base.ParserTestBase):
 
   def test_assign_namedtuple(self):
     self.check("X = NamedTuple('X', [])", """
-      from typing import Any, Tuple, Type, TypeVar
+      from typing import NamedTuple
 
       X = namedtuple_X_0
 
-      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
-
-      class namedtuple_X_0(Tuple[()]):
-          __slots__ = []
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_X_0]) -> _Tnamedtuple_X_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
+      class namedtuple_X_0(NamedTuple): ...
     """)
 
   def test_subclass_namedtuple(self):
-    self.check("class X(NamedTuple('X', [])): ...", """
-      from typing import Any, Tuple, Type, TypeVar
+    self.check("""
+      from typing import NamedTuple
+      class X(NamedTuple('X', [])): ...
+    """, """
+      from typing import NamedTuple
 
-      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
-
-      class namedtuple_X_0(Tuple[()]):
-          __slots__ = []
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_X_0]) -> _Tnamedtuple_X_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
+      class namedtuple_X_0(NamedTuple): ...
 
       class X(namedtuple_X_0): ...
     """)
@@ -727,25 +723,13 @@ class NamedTupleTest(parser_test_base.ParserTestBase):
           ],
       )
     """, """
-      from typing import Any, Tuple, Type, TypeVar
+      from typing import NamedTuple
 
       Foo = namedtuple_Foo_0
 
-      _Tnamedtuple_Foo_0 = TypeVar('_Tnamedtuple_Foo_0', bound=namedtuple_Foo_0)
-
-      class namedtuple_Foo_0(Tuple[int, str]):
-          __slots__ = ["a", "b"]
+      class namedtuple_Foo_0(NamedTuple):
           a: int
           b: str
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_Foo_0], a: int, b: str) -> _Tnamedtuple_Foo_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
     """)
 
   def test_collections_trailing_comma(self):
@@ -760,48 +744,24 @@ class NamedTupleTest(parser_test_base.ParserTestBase):
       )
     """, """
       from collections import namedtuple
-      from typing import Any, Tuple, Type, TypeVar
+      from typing import Any, NamedTuple
 
       Foo = namedtuple_Foo_0
 
-      _Tnamedtuple_Foo_0 = TypeVar('_Tnamedtuple_Foo_0', bound=namedtuple_Foo_0)
-
-      class namedtuple_Foo_0(Tuple[Any, Any]):
-          __slots__ = ["a", "b"]
+      class namedtuple_Foo_0(NamedTuple):
           a: Any
           b: Any
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_Foo_0], a, b) -> _Tnamedtuple_Foo_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
     """)
 
   def test_collections_namedtuple(self):
     expected = """
       from collections import namedtuple
-      from typing import Any, Tuple, Type, TypeVar
+      from typing import Any, NamedTuple
 
       X = namedtuple_X_0
 
-      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
-
-      class namedtuple_X_0(Tuple[Any]):
-          __slots__ = ["y"]
+      class namedtuple_X_0(NamedTuple):
           y: Any
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_X_0], y) -> _Tnamedtuple_X_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
     """
     self.check("""
       from collections import namedtuple
@@ -819,25 +779,11 @@ class NamedTupleTest(parser_test_base.ParserTestBase):
         y: int
         z: str
     """, """
-      from typing import Any, Tuple, Type, TypeVar
+      from typing import NamedTuple
 
-      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
-
-      class namedtuple_X_0(Tuple[int, str]):
-          __slots__ = ["y", "z"]
+      class X(NamedTuple):
           y: int
           z: str
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_X_0], y: int, z: str) -> _Tnamedtuple_X_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
-
-      class X(namedtuple_X_0): ...
     """)
 
   def test_typing_namedtuple_class_with_method(self):
@@ -848,25 +794,11 @@ class NamedTupleTest(parser_test_base.ParserTestBase):
         z: str
         def foo(self) -> None: ...
     """, """
-      from typing import Any, Tuple, Type, TypeVar
+      from typing import NamedTuple
 
-      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
-
-      class namedtuple_X_0(Tuple[int, str]):
-          __slots__ = ["y", "z"]
+      class X(NamedTuple):
           y: int
           z: str
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_X_0], y: int, z: str) -> _Tnamedtuple_X_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
-
-      class X(namedtuple_X_0):
           def foo(self) -> None: ...
     """)
 
@@ -877,25 +809,11 @@ class NamedTupleTest(parser_test_base.ParserTestBase):
         y: int
         z: str
     """, """
-      from typing import Any, Tuple, Type, TypeVar
+      from typing import NamedTuple
 
-      _Tnamedtuple_X_0 = TypeVar('_Tnamedtuple_X_0', bound=namedtuple_X_0)
-
-      class namedtuple_X_0(Tuple[int, str]):
-          __slots__ = ["y", "z"]
+      class X(dict, NamedTuple):
           y: int
           z: str
-          _asdict: Any
-          __dict__: Any
-          _fields: Any
-          __getnewargs__: Any
-          __getstate__: Any
-          _make: Any
-          _replace: Any
-          def __new__(cls: Type[_Tnamedtuple_X_0], y: int, z: str) -> _Tnamedtuple_X_0: ...
-          def __init__(self, *args, **kwargs) -> None: ...
-
-      class X(dict, namedtuple_X_0): ...
     """)
 
   def test_multi_namedtuple_base(self):
@@ -994,7 +912,7 @@ class FunctionTest(parser_test_base.ParserTestBase):
           bar: str = ...  # type: ignore
     """, """
       class Foo:
-          bar: str
+          bar: str = ...
     """)
     self.check("""
       def f(  # type: ignore
@@ -1190,15 +1108,8 @@ class FunctionTest(parser_test_base.ParserTestBase):
 
       @dataclass
       class Foo:
-        x: int
-        y: str = ...
-    """, """
-    from dataclasses import dataclass
-
-    @dataclass
-    class Foo:
-        x: int
-        y: str
+          x: int
+          y: str = ...
     """)
 
   def test_dataclass_default_error(self):
@@ -2519,7 +2430,7 @@ class LiteralTest(parser_test_base.ParserTestBase):
       x6: Final = Color.RED
     """, """
       import enum
-      from typing import Any, Literal
+      from typing import Literal
       from typing_extensions import Final
 
       x1: Literal[3]
@@ -2543,7 +2454,7 @@ class LiteralTest(parser_test_base.ParserTestBase):
       from typing_extensions import Final
 
       x: typing_extensions.Final
-      y: typing_extensions.Final
+      y: typing_extensions.Final = ...
     """)
 
   def test_bad_final_literal(self):
